//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#include "ffl.h"
#include "Spectrum.h"

using namespace std;

int main (int argc, char* argv[]){

  // check the argument
  if(argc!=7){
    cerr<<""<<endl;
    cerr<<"usage:"<<endl;
    cerr<<argv[0]<<" [ffl file] [channel name] [gps start] [gps end] [PSD frequency resolution] [N subsegments]"<<endl; 
    cerr<<""<<endl;
    return 1;
  }

  // channel name
  string channelname = (string)argv[2];

  // time range
  int gps_start = atoi(argv[3]);
  int gps_end = atoi(argv[4]);
  int timerange=gps_end-gps_start;

  // load ffl file
  string fflfile = (string)argv[1];
  ffl *FFL = new ffl(fflfile);
  if(!FFL->LoadFrameFile(gps_start)) return 2;

  // channel sampling frequency
  int dummy;
  int channelsampling = FFL->GetChannelSampling(channelname,dummy);
  if(!channelsampling) return 3;

  // PSD resolution
  double df = atof(argv[5]);
  int N = NextPowerOfTwo((double)channelsampling/2.0/df);

  // data vector
  int nsub = atoi(argv[6]);
  int dataduration = nsub*2*N/channelsampling;
  if(dataduration>timerange){
    cerr<<"the input time range is not long enough to get such a high PSD frequency resolution"<<endl;
    return 4;
  }


  // Spectrum
  Spectrum *S = new Spectrum(N,timerange,channelsampling,2);

  // data container
  double *data;
  
  int tstart=gps_start;
  int tend=gps_start+dataduration;
  int vsize;
  while(tend<gps_end){
    cout<<tstart<<" - "<<tend<<endl;
    
    // get data
    data=FFL->GetData(vsize,channelname,tstart,tend);
    if(vsize!=dataduration*channelsampling) return 5;
    
    S->AddData(vsize,data);
    delete data;
    
    // next chunk
    tstart=tend;
    tend=tstart+dataduration;
  }

  TGraph *P = S->GetPSD();
  TGraph *A = S->GetASD();

  stringstream ss;
  ss<<"mypsd_"<<ReplaceAll(channelname,":","-")<<"-"<<gps_start<<"-"<<tstart-gps_start<<".root";
  TFile *fout = new TFile(ss.str().c_str(),"recreate");
  fout->cd();
  P->Write();
  A->Write();
  fout->Close();

  delete P;
  delete A;
  delete S;
  delete FFL;
  
  return 0;
}

