//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#include "Triggers.h"

////////////////////////////////////////////////////////////////////////////////////
void Triggers::ResetClusters(void){
////////////////////////////////////////////////////////////////////////////////////
  Cmeandur  = 0.0;
  Cmaxdur   = 0.0;
  Ctotdur   = 0.0;
  delete cl_tag;
  cl_tag = new int [0];
  Ctree->Reset();
  Ctend=0.0;
  Cph=-100.0;// FIXME: should be removed
  Cfirstentry=-1;
}

////////////////////////////////////////////////////////////////////////////////////
bool Triggers::Clusterize(const int aTag){ 
////////////////////////////////////////////////////////////////////////////////////

  // some printing
  if(Verbose) cout<<"Triggers::Clusterize: cluster "<<Ttree->GetEntries()<<" triggers..."<<endl;
  if(Verbose>1){
    cout<<"   > delta_t          = "<<Cdelta_t<<endl;
    cout<<"   > size min         = "<<Csizemin<<endl;
    cout<<"   > SNR threshold    = "<<cl_snr_thr<<endl;
  }

  // reset any previous clustering
  ResetClusters();

  // clusterize
  if(!Clusterize_t()){
      ResetClusters();
      return false;
  }

  // number of clusters
  if(Verbose>1) cout<<"Triggers::Clusterize: "<<Ctree->GetEntries()<<" clusters were found"<<endl;
  if(Verbose>2){
    cout<<"Triggers::Clusterize: cluster mean duration = "<<Cmeandur<<" s"<<endl;
    cout<<"Triggers::Clusterize: cluster max duration = "<<Cmaxdur<<" s"<<endl;
    cout<<"Triggers::Clusterize: cluster total duration = "<<Ctotdur<<" s"<<endl;
  }
  
  // init tag vector
  delete cl_tag;
  cl_tag = new int [Ctree->GetEntries()];
  for(int c=0; c<Ctree->GetEntries(); c++) cl_tag[c]=aTag;
  
  return true;
}

////////////////////////////////////////////////////////////////////////////////////
bool Triggers::Clusterize_t(void){
////////////////////////////////////////////////////////////////////////////////////
   
  // loop over time sorted triggers
  for(Long64_t t=0; t<Ttree->GetEntries(); t++){
    Ttree->GetEntry(t);
    
    // this is the same cluster...
    if(Ttstart-Ctend<=Cdelta_t){
      Csize++; // one more tile
      if(Ttend   > Ctend)   Ctend=Ttend;    // update cluster end
      if(Ttstart < Ctstart) Ctstart=Ttstart;// update cluster tstart
      if(Tfend   > Cfend)   Cfend=Tfend;    // update cluster end
      if(Tfstart < Cfstart) Cfstart=Tfstart;// update cluster tstart
      if(Tsnr>Csnr){  // this tile is louder
        Csnr  = Tsnr; // update cluster SNR
        Camp  = Tamp; // update cluster amplitude - FIXME: could be better!
        Cph   = Tph;  // update cluster phase
        Ctime = Ttime;// update cluster time
        Cfreq = Tfreq;// update cluster frequency
        Cq    = Tq;   // update cluster Q
      }
    }
    //... or start a new cluster
    else{
      if(t&&Csnr>=cl_snr_thr&&Csize>=Csizemin){
        Ctree->Fill();  // fill tree with previous entry
        Cmeandur+=(Ctend-Ctstart);
        if(Ctend-Ctstart>Cmaxdur) Cmaxdur=Ctend-Ctstart;
      }
      Ctend       = Ttend;  // cluster t end
      Ctstart     = Ttstart;// cluster t start
      Cfend       = Tfend;  // cluster f end
      Cfstart     = Tfstart;// cluster f start
      Csize       = 1;      // cluster size
      Csnr        = Tsnr;   // cluster SNR
      Camp        = Tamp;   // cluster amplitude
      Ctime       = Ttime;  // cluster time
      Cfreq       = Tfreq;  // cluster frequency
      Cq          = Tq;     // cluster Q
      Cph         = Tph;    // cluster phase
      Cfirstentry = t;      // cluster first entry
    }
  }

  // save last cluster
  if(Ttree->GetEntries()&&Csnr>=cl_snr_thr&&Csize>=Csizemin){
    Ctree->Fill();
    Cmeandur+=(Ctend-Ctstart);
    if(Ctend-Ctstart>Cmaxdur) Cmaxdur=Ctend-Ctstart;
  }

  // cluster stats
  Ctotdur=Cmeandur;
  if(Ctree->GetEntries()) Cmeandur/=(double)(Ctree->GetEntries());
   
  return true;
}

////////////////////////////////////////////////////////////////////////////////////
bool Triggers::SetClusterTag(const int aCl, const int aNewTag){ 
////////////////////////////////////////////////////////////////////////////////////
  if(aCl<0||aCl>=Ctree->GetEntries()){
    cerr<<"Triggers::SetClusterTag: the cluster number does not exist: "<<aCl<<endl;
    return false;
  }
  cl_tag[aCl]=aNewTag;
  return true;
}

////////////////////////////////////////////////////////////////////////////////////
bool Triggers::SetClusterBranchStatus(const string aBname, const bool aStatus){ 
////////////////////////////////////////////////////////////////////////////////////
  UInt_t found;
  Ctree->SetBranchStatus(aBname.c_str(), aStatus, &found);
  return (bool)found;
}

////////////////////////////////////////////////////////////////////////////////////
int Triggers::GetClusterIndex(const double aTime){ 
////////////////////////////////////////////////////////////////////////////////////
  // loop over clusters
  for(int c=0; c<Ctree->GetEntries(); c++){
    Ctree->GetEntry(c);
    if(aTime>=Ctstart&&aTime<Ctend) return c;
  }
  return -1; // not found
}


