//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#include "TriggerPlot.h"

ClassImp(TriggerPlot)

////////////////////////////////////////////////////////////////////////////////////
TriggerPlot::TriggerPlot(const int aNcoll, const string aPattern, const string aDirectory, const string aStyleName, const int aVerbose) : GwollumPlot ("TP",aStyleName), ReadTriggers (aPattern, aDirectory, aVerbose){ 
////////////////////////////////////////////////////////////////////////////////////

  Ncoll=(int)fabs(aNcoll);

  // allocate memory (style)
  TPcolor         = new int            [Ncoll];
  TPmsize         = new double         [Ncoll];
  TPlsize         = new int            [Ncoll];
  TPmstyle        = new int            [Ncoll];
  Slegend         = new string         [Ncoll];

  // allocate memory (selection)
  CollectionParam = new TriggerSelect* [Ncoll];
  Suseclusters    = new int            [Ncoll];
  ev_duration_tot = new double         [Ncoll];

  // allocate memory (plots)
  Hfreqtime       = new TH2D*          [Ncoll];
  Gfreqtime       = new TGraph*        [Ncoll];
  Hsnrtime        = new TH2D*          [Ncoll];
  Gsnrtime        = new TGraph*        [Ncoll];
  Hsnrfreq        = new TH2D*          [Ncoll];
  Gsnrfreq        = new TGraph*        [Ncoll];
  Htime           = new TH1D*          [Ncoll];
  Hfreq           = new TH1D*          [Ncoll];
  Hsnr            = new TH1D*          [Ncoll];
  Hproc           = new TH1D();

  for(int p=0; p<Ncoll; p++){
    CollectionParam[p] = new TriggerSelect(this,ReadTriggerSegments::Verbose);
    Hfreqtime[p]   = CollectionParam[p]->GetTH2D("Time", "Frequency");
    Gfreqtime[p]   = new TGraph();
    Hsnrtime[p]    = CollectionParam[p]->GetTH2D("Time", "SNR");
    Gsnrtime[p]    = new TGraph();
    Hsnrfreq[p]    = CollectionParam[p]->GetTH2D("Frequency", "SNR");
    Gsnrfreq[p]    = new TGraph();
    Htime[p]       = CollectionParam[p]->GetTH1D("Time");
    Hfreq[p]       = CollectionParam[p]->GetTH1D("Frequency");
    Hsnr[p]        = CollectionParam[p]->GetTH1D("SNR");
    ResetParameters(p);
    ev_duration_tot[p]=0.0;
  }

  // loudest event
  plot_star=true;
  Gfreqtimeloud   = new TGraph();
  Gfreqtimeloud2  = new TGraph();
  Gfreqtimeloud->SetMarkerStyle(29);
  Gfreqtimeloud->SetMarkerSize(2);
  Gfreqtimeloud->SetMarkerColor(5);
  Gfreqtimeloud2->SetMarkerStyle(30);
  Gfreqtimeloud2->SetMarkerSize(2);
  Gfreqtimeloud2->SetMarkerColor(1);

  // segment box
  SegBox = new TPave(0.1,0.905,0.84,0.925,0,"NDC");
  SegBox->SetFillColor(2);
  nonbox=0;
  OnBox = new TPave* [0];
  nselbox=0;
  SelBox = new TPave* [0];
  SetTimeBox();// draw time box by default

  // selection segments
  SelSeg = NULL;
  
}

////////////////////////////////////////////////////////////////////////////////////
TriggerPlot::~TriggerPlot(void){
////////////////////////////////////////////////////////////////////////////////////
  if(ReadTriggerSegments::Verbose>1) cout<<"TriggerPlot::~TriggerPlot"<<endl;
  delete TPcolor;
  delete TPmsize;
  delete TPlsize;
  delete TPmstyle;
  delete [] Slegend;
  for(int p=0; p<Ncoll; p++){
    delete Hfreqtime[p];
    delete Gfreqtime[p];
    delete Hsnrtime[p];
    delete Gsnrtime[p];
    delete Hsnrfreq[p];
    delete Gsnrfreq[p];
    delete Htime[p];
    delete Hfreq[p];
    delete Hsnr[p];
    delete CollectionParam[p];
  }
  delete Hfreqtime;
  delete Gfreqtime;
  delete Gfreqtimeloud;
  delete Gfreqtimeloud2;
  delete Hsnrtime;
  delete Gsnrtime;
  delete Hsnrfreq;
  delete Gsnrfreq;
  delete Htime;
  delete Hfreq;
  delete Hsnr;
  delete Hproc;
  delete CollectionParam;
  delete SegBox;
  for(int i=0; i<nonbox; i++) delete OnBox[i];
  delete OnBox;
  for(int i=0; i<nselbox; i++) delete SelBox[i];
  delete SelBox;
  delete ev_duration_tot;
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerPlot::ResetParameters(const int aCollIndex){
////////////////////////////////////////////////////////////////////////////////////
  CollectionParam[aCollIndex]->ResetSelection();  // back to default trigger selection
  CollectionParam[aCollIndex]->SetTimeResolution(TMath::Min(TMath::Max(1,GetNTriggers()/500),100));
  CollectionParam[aCollIndex]->SetFrequencyResolution(TMath::Min(TMath::Max(1,GetNTriggers()/500),100));
  CollectionParam[aCollIndex]->SetSNRResolution(TMath::Min(TMath::Max(1,GetNTriggers()/500),100));
  
  Pusedate=false;                                        // use GPS measure
  
  ostringstream tmpstream;
  tmpstream<<aCollIndex;
  TPcolor[aCollIndex]=1+aCollIndex;                      // default colors
  TPmsize[aCollIndex]=1.0;                               // default marker size
  TPlsize[aCollIndex]=2;                                 // default line size
  TPmstyle[aCollIndex]=1;                                // marker = dots
  Slegend[aCollIndex]="Collection #"+tmpstream.str();    // default legend

  Suseclusters[aCollIndex]=-1;                           // no clustering
  tmpstream.str(""); tmpstream.clear();
  
  return ResetCollection(aCollIndex);
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerPlot::ApplyStyle(const int aCollIndex){
////////////////////////////////////////////////////////////////////////////////////
 
  // time offset
  double offset=EPOCH_UNIX_GPS - LeapSeconds(CollectionParam[aCollIndex]->GetTimeMin()) + EPOCH_GPS_TAI_UTC;
 
  // timing format
  if(Pusedate){
    Hfreqtime[aCollIndex]->GetXaxis()->SetTimeFormat("");
    Hfreqtime[aCollIndex]->GetXaxis()->SetTimeDisplay(1);
    Hfreqtime[aCollIndex]->GetXaxis()->SetTimeOffset(offset,"gmt");
    Hfreqtime[aCollIndex]->GetXaxis()->SetTitle("Date");
    if(Suseclusters[aCollIndex]>=0) Hfreqtime[aCollIndex]->SetTitle((GetStreamName()+": cluster frequency vs. time").c_str());
    else Hfreqtime[aCollIndex]->SetTitle((GetStreamName()+" trigger frequency vs. time").c_str());
    Hsnrtime[aCollIndex]->GetXaxis()->SetTimeFormat("");
    Hsnrtime[aCollIndex]->GetXaxis()->SetTimeDisplay(1);
    Hsnrtime[aCollIndex]->GetXaxis()->SetTimeOffset(offset,"gmt");
    Hsnrtime[aCollIndex]->GetXaxis()->SetTitle("Date");
    if(Suseclusters[aCollIndex]>=0) Hsnrtime[aCollIndex]->SetTitle((GetStreamName()+": cluster SNR vs. time").c_str());
    else Hsnrtime[aCollIndex]->SetTitle((GetStreamName()+": trigger SNR vs. time").c_str());
    Htime[aCollIndex]->GetXaxis()->SetTimeFormat("");
    Htime[aCollIndex]->GetXaxis()->SetTimeDisplay(1);
    Htime[aCollIndex]->GetXaxis()->SetTimeOffset(offset,"gmt");
    Htime[aCollIndex]->GetXaxis()->SetTitle("Date");
    if(Suseclusters[aCollIndex]>=0) Htime[aCollIndex]->SetTitle((GetStreamName()+": cluster rate").c_str());
    else Htime[aCollIndex]->SetTitle((GetStreamName()+" trigger rate").c_str());
  }
  else{
    Hfreqtime[aCollIndex]->GetXaxis()->SetTimeFormat("");
    Hfreqtime[aCollIndex]->GetXaxis()->SetTimeDisplay(0);
    if(Suseclusters[aCollIndex]>=0) Hfreqtime[aCollIndex]->SetTitle((GetStreamName()+": cluster frequency vs. time").c_str());
    else Hfreqtime[aCollIndex]->SetTitle((GetStreamName()+": trigger frequency vs. time").c_str());
    Hfreqtime[aCollIndex]->GetXaxis()->SetTitle("Time [s]");
    Hsnrtime[aCollIndex]->GetXaxis()->SetTimeFormat("");
    Hsnrtime[aCollIndex]->GetXaxis()->SetTimeDisplay(0);
    if(Suseclusters[aCollIndex]>=0) Hsnrtime[aCollIndex]->SetTitle((GetStreamName()+": cluster SNR vs. time").c_str());
    else  Hsnrtime[aCollIndex]->SetTitle((GetStreamName()+": trigger SNR vs. time").c_str());
    Htime[aCollIndex]->GetXaxis()->SetTitle("Time [s]");
    Htime[aCollIndex]->GetXaxis()->SetTimeFormat("");
    Htime[aCollIndex]->GetXaxis()->SetTimeDisplay(0);
    if(Suseclusters[aCollIndex]>=0) Htime[aCollIndex]->SetTitle((GetStreamName()+": cluster rate").c_str());
    else  Htime[aCollIndex]->SetTitle((GetStreamName()+": trigger rate").c_str());
    Htime[aCollIndex]->GetXaxis()->SetTitle("Time [s]");
  }

  // marker
  Htime[aCollIndex]->SetMarkerStyle(TPmstyle[aCollIndex]);
  Htime[aCollIndex]->SetMarkerSize(TPmsize[aCollIndex]); 
  Htime[aCollIndex]->SetMarkerColor(TPcolor[aCollIndex]);
  Gfreqtime[aCollIndex]->SetMarkerStyle(TPmstyle[aCollIndex]);
  Gfreqtime[aCollIndex]->SetMarkerSize(TPmsize[aCollIndex]);
  Gfreqtime[aCollIndex]->SetMarkerColor(TPcolor[aCollIndex]);
  Gsnrtime[aCollIndex]->SetMarkerStyle(TPmstyle[aCollIndex]);
  Gsnrtime[aCollIndex]->SetMarkerSize(TPmsize[aCollIndex]);
  Gsnrtime[aCollIndex]->SetMarkerColor(TPcolor[aCollIndex]);
  Gsnrfreq[aCollIndex]->SetMarkerStyle(TPmstyle[aCollIndex]);
  Gsnrfreq[aCollIndex]->SetMarkerSize(TPmsize[aCollIndex]);
  Gsnrfreq[aCollIndex]->SetMarkerColor(TPcolor[aCollIndex]);

  // line
  Htime[aCollIndex]->SetLineWidth(TPlsize[aCollIndex]);
  Htime[aCollIndex]->SetLineColor(TPcolor[aCollIndex]);
  Hfreq[aCollIndex]->SetLineWidth(TPlsize[aCollIndex]);
  Hfreq[aCollIndex]->SetLineColor(TPcolor[aCollIndex]);
  Hsnr[aCollIndex]->SetLineWidth(TPlsize[aCollIndex]);
  Hsnr[aCollIndex]->SetLineColor(TPcolor[aCollIndex]);
  Gfreqtime[aCollIndex]->SetLineWidth(TPlsize[aCollIndex]);
  Gfreqtime[aCollIndex]->SetLineColor(TPcolor[aCollIndex]);
  Gsnrtime[aCollIndex]->SetLineWidth(TPlsize[aCollIndex]);
  Gsnrtime[aCollIndex]->SetLineColor(TPcolor[aCollIndex]);
  Gsnrfreq[aCollIndex]->SetLineWidth(TPlsize[aCollIndex]);
  Gsnrfreq[aCollIndex]->SetLineColor(TPcolor[aCollIndex]);

  return;
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerPlot::ResetCollection(const int aCollIndex){
////////////////////////////////////////////////////////////////////////////////////
  ev_duration_tot[aCollIndex]=0.0;
  
  //********************************************************************************
  //                                 FREQTIME
  //********************************************************************************
  delete Hfreqtime[aCollIndex];
  Hfreqtime[aCollIndex] = CollectionParam[aCollIndex]->GetTH2D("Time", "Frequency");
  delete Gfreqtime[aCollIndex];
  Gfreqtime[aCollIndex]=new TGraph();

  ostringstream tmpstream;
  tmpstream<<"Hfreqtime_"<<aCollIndex<<"_"<<ReadTriggerSegments::srandid;
  Hfreqtime[aCollIndex]->SetName(tmpstream.str().c_str());
  tmpstream.str(""); tmpstream.clear();
  tmpstream<<"Gfreqtime_"<<aCollIndex<<"_"<<ReadTriggerSegments::srandid;
  Gfreqtime[aCollIndex]->SetName(tmpstream.str().c_str());
  tmpstream.str(""); tmpstream.clear();

  Hfreqtime[aCollIndex]->GetXaxis()->SetTitle("Time [s]");
  Hfreqtime[aCollIndex]->GetYaxis()->SetTitle("Frequency [Hz]");
  Hfreqtime[aCollIndex]->GetXaxis()->SetNoExponent();

  //********************************************************************************
  //                                  SNRTIME
  //********************************************************************************
  delete Hsnrtime[aCollIndex];
  Hsnrtime[aCollIndex] = CollectionParam[aCollIndex]->GetTH2D("Time", "SNR");
  if(Hsnrtime[aCollIndex]==NULL) Hsnrtime[aCollIndex] = new TH2D();
  delete Gsnrtime[aCollIndex];
  Gsnrtime[aCollIndex]=new TGraph();

  tmpstream<<"Hsnrtime_"<<aCollIndex<<"_"<<ReadTriggerSegments::srandid;
  Hsnrtime[aCollIndex]->SetName(tmpstream.str().c_str());
  tmpstream.str(""); tmpstream.clear();
  tmpstream<<"Gsnrtime_"<<aCollIndex<<"_"<<ReadTriggerSegments::srandid;
  Gsnrtime[aCollIndex]->SetName(tmpstream.str().c_str());
  tmpstream.str(""); tmpstream.clear();

  Hsnrtime[aCollIndex]->GetXaxis()->SetTitle("Time [s]");
  Hsnrtime[aCollIndex]->GetYaxis()->SetTitle("SNR [-]");
  Hsnrtime[aCollIndex]->GetXaxis()->SetNoExponent();

  //********************************************************************************
  //                                  SNRFREQ
  //********************************************************************************
  delete Hsnrfreq[aCollIndex];
  Hsnrfreq[aCollIndex] = CollectionParam[aCollIndex]->GetTH2D("Frequency", "SNR");
  if(Hsnrfreq[aCollIndex]==NULL) Hsnrfreq[aCollIndex] = new TH2D();
  delete Gsnrfreq[aCollIndex];
  Gsnrfreq[aCollIndex]=new TGraph();

  tmpstream<<"Hsnrfreq_"<<aCollIndex<<"_"<<ReadTriggerSegments::srandid;
  Hsnrfreq[aCollIndex]->SetName(tmpstream.str().c_str());
  tmpstream.str(""); tmpstream.clear();
  tmpstream<<"Gsnrfreq_"<<aCollIndex<<"_"<<ReadTriggerSegments::srandid;
  Gsnrfreq[aCollIndex]->SetName(tmpstream.str().c_str());
  tmpstream.str(""); tmpstream.clear();

  Hsnrfreq[aCollIndex]->GetXaxis()->SetTitle("Frequency [Hz]");
  Hsnrfreq[aCollIndex]->GetYaxis()->SetTitle("SNR [-]");
  if(Suseclusters[aCollIndex]>=0) Hsnrfreq[aCollIndex]->SetTitle((GetStreamName()+": Cluster SNR vs frequency").c_str());
  else Hsnrfreq[aCollIndex]->SetTitle((GetStreamName()+": Trigger SNR vs frequency").c_str());
  
  //********************************************************************************
  //                                    TIME
  //********************************************************************************
  delete Htime[aCollIndex];
  TPlivetime=0.0;
  Htime[aCollIndex] = CollectionParam[aCollIndex]->GetTH1D("Time");
  if(Htime[aCollIndex]==NULL) Htime[aCollIndex] = new TH1D();

  tmpstream<<"Htime_"<<aCollIndex<<"_"<<ReadTriggerSegments::srandid;
  Htime[aCollIndex]->SetName(tmpstream.str().c_str());
  tmpstream.str(""); tmpstream.clear();
  
  Htime[aCollIndex]->GetXaxis()->SetTitle("Time [s]");
  tmpstream<<Htime[aCollIndex]->GetXaxis()->GetBinWidth(0);
  Htime[aCollIndex]->GetXaxis()->SetNoExponent();
  if(Suseclusters[aCollIndex]>=0){
    Htime[aCollIndex]->GetYaxis()->SetTitle(("Cluster rate [Hz] (avg over "+tmpstream.str()+" s)").c_str());
    Htime[aCollIndex]->SetTitle((GetStreamName()+": cluster rate").c_str());
  }
  else{
    Htime[aCollIndex]->GetYaxis()->SetTitle(("Trigger rate [Hz] (avg over "+tmpstream.str()+" s)").c_str());
    Htime[aCollIndex]->SetTitle((GetStreamName()+": trigger rate").c_str());
  }
  tmpstream.str(""); tmpstream.clear();

  //********************************************************************************
  //                                    FREQUENCY
  //********************************************************************************
  delete Hfreq[aCollIndex];
  Hfreq[aCollIndex] = CollectionParam[aCollIndex]->GetTH1D("Frequency");
  if(Hfreq[aCollIndex]==NULL) Hfreq[aCollIndex] = new TH1D();

  tmpstream<<"Hfreq_"<<aCollIndex<<"_"<<ReadTriggerSegments::srandid;
  Hfreq[aCollIndex]->SetName(tmpstream.str().c_str());
  tmpstream.str(""); tmpstream.clear();

  Hfreq[aCollIndex]->GetXaxis()->SetTitle("Frequency [Hz]");
  if(Suseclusters[aCollIndex]>=0){
    Hfreq[aCollIndex]->GetYaxis()->SetTitle("Number of clusters / bin");
    Hfreq[aCollIndex]->SetTitle((GetStreamName()+": cluster frequency distribution").c_str());
  }
  else{
    Hfreq[aCollIndex]->GetYaxis()->SetTitle("Number of triggers / bin");
    Hfreq[aCollIndex]->SetTitle((GetStreamName()+": trigger frequency distribution").c_str());
  }
  
  //********************************************************************************
  //                                    SNR
  //********************************************************************************
  delete Hsnr[aCollIndex];
  Hsnr[aCollIndex] = CollectionParam[aCollIndex]->GetTH1D("SNR");
  if(Hsnr[aCollIndex]==NULL) Hsnr[aCollIndex] = new TH1D();

  tmpstream<<"Hsnr_"<<aCollIndex<<"_"<<ReadTriggerSegments::srandid;
  Hsnr[aCollIndex]->SetName(tmpstream.str().c_str());
  tmpstream.str(""); tmpstream.clear();
  
  Hsnr[aCollIndex]->GetXaxis()->SetTitle("SNR [-]");
  if(Suseclusters[aCollIndex]>=0){
    Hsnr[aCollIndex]->GetYaxis()->SetTitle("Number of clusters / bin");
    Hsnr[aCollIndex]->SetTitle((GetStreamName()+": cluster SNR distribution").c_str());
  }
  else{
      Hsnr[aCollIndex]->GetYaxis()->SetTitle("Number of triggers / bin");
      Hsnr[aCollIndex]->SetTitle((GetStreamName()+": trigger SNR distribution").c_str());
  }
 
  if(ReadTriggerSegments::Verbose>1){
    cout<<"EventPlot::ResetCollection: Collection #"<<aCollIndex<<" was created with the following selection:"<<endl;
    CollectionParam[aCollIndex]->PrintSelection();
  }

  return ApplyStyle(aCollIndex);
}

////////////////////////////////////////////////////////////////////////////////////
bool TriggerPlot::MakeCollections(const int aCollIndex, Segments *aInSeg){
////////////////////////////////////////////////////////////////////////////////////

  double collectiontmin;
  double collectiontmax;
  double collectionfmin;
  double collectionfmax;
  double collectionsmin;
  double collectionsmax;
  int nc;
  double tmax=-1;
  double fmax=-1;
  double smax=-1;

  int istart=aCollIndex;
  int istop=aCollIndex+1;
  if(aCollIndex<0||aCollIndex>=Ncoll){
    istart=0;
    istop=Ncoll;
  }

  // loop over collections
  for(int i=istart; i<istop; i++){
    ResetCollection(i);
    if(ReadTriggerSegments::Verbose>1) cout<<"TriggerPlot::MakeCollections: make collection #"<<i<<endl;

    // selection
    collectiontmin=CollectionParam[i]->GetTimeMin();
    collectiontmax=CollectionParam[i]->GetTimeMax();
    collectionfmin=CollectionParam[i]->GetFrequencyMin();
    collectionfmax=CollectionParam[i]->GetFrequencyMax();
    collectionsmin=CollectionParam[i]->GetSNRMin();
    collectionsmax=CollectionParam[i]->GetSNRMax();

    nc=0;
    
    // clusters
    if(Suseclusters[i]>=0){
      for(int c=0; c<Ctree->GetEntries(); c++){
	if(!Suseclusters[i]){// tag selection
	  if(cl_tag[c]<0) continue;
	}
	else{
	  if(cl_tag[c]!=Suseclusters[i]) continue;
	  //if(cl_tag[c]<Suseclusters[i]) continue;
	}
	Ctree->GetEntry(c);
	
	// apply selection
	if(Ctime<collectiontmin) continue;
	if(Ctime>=collectiontmax) continue;
	if(Cfreq<collectionfmin) continue;
	if(Cfreq>=collectionfmax) continue;
	if(Csnr<collectionsmin) continue;
	if(Csnr>=collectionsmax) continue;
	if(Ctstart>=collectiontmax) break;
	if(SelSeg!=NULL){
	  if(!SelSeg->IsInsideSegment(Ctime)) continue;
	}
	
	Gfreqtime[i]->SetPoint(nc, Ctime, Cfreq);
	Gsnrtime[i]->SetPoint(nc, Ctime, Csnr);
	Gsnrfreq[i]->SetPoint(nc, Cfreq, Csnr);
	Htime[i]->Fill(Ctime);
	Hfreq[i]->Fill(Cfreq);
	Hsnr[i]->Fill(Csnr);
	nc++;
	ev_duration_tot[i]+=Ctend-Ctstart;
	if(Csnr>smax){ tmax=Ctime; fmax=Cfreq; smax=Csnr; }
      }
    }
    
    // triggers
    else{
      for(int t=0; t<Ttree->GetEntries(); t++){
	Ttree->GetEntry(t);
	
	// apply selection
	if(Ttime<collectiontmin) continue;
	if(Ttime>=collectiontmax) continue;
	if(Tfreq<collectionfmin) continue;
	if(Tfreq>=collectionfmax) continue;
	if(Tsnr<collectionsmin) continue;
	if(Tsnr>=collectionsmax) continue;
	if(Ttstart>=collectiontmax) break;
	if(SelSeg!=NULL){
	  if(!SelSeg->IsInsideSegment(Ttime)) continue;
	}

	Gfreqtime[i]->SetPoint(nc, Ttime, Tfreq);
	Gsnrtime[i]->SetPoint(nc, Ttime, Tsnr);
	Gsnrfreq[i]->SetPoint(nc, Tfreq, Tsnr);
	Htime[i]->Fill(Ttime);
	Hfreq[i]->Fill(Tfreq);
	Hsnr[i]->Fill(Tsnr);
	nc++;
	ev_duration_tot[i]+=Ttend-Ttstart;
	if(Tsnr>smax){ tmax=Ttime; fmax=Tfreq; smax=Tsnr; }
      }
    }
  
    // time -> rate
    double bintime;
    TPlivetime=0.0;
    for(int b=1; b<=Htime[i]->GetNbinsX(); b++){
      bintime=Seg->GetLiveTime(Htime[i]->GetXaxis()->GetBinLowEdge(b),Htime[i]->GetXaxis()->GetBinUpEdge(b));
      if(bintime) Htime[i]->SetBinContent(b,Htime[i]->GetBinContent(b)/bintime);
      TPlivetime+=bintime;
    }
         
    // segment boxes
    if(i==0){

      // ON
      for(int j=0; j<nonbox; j++) delete OnBox[j];
      nonbox=0; delete OnBox;
      double min=CollectionParam[0]->GetTimeMin();
      double max=CollectionParam[0]->GetTimeMax();
      Segments *OnSeg = new Segments(min,max);
      if(aInSeg!=NULL&&aInSeg->GetStatus()) OnSeg->Intersect(aInSeg);
      OnSeg->Intersect(ReadTriggerSegments::Seg);
      OnBox = new TPave* [OnSeg->GetNsegments()];
      for(int s=0; s<OnSeg->GetNsegments(); s++){
	OnBox[s]= new TPave(SegBox->GetX1()+(SegBox->GetX2()-SegBox->GetX1())/(max-min)*(OnSeg->GetStart(s)-min),SegBox->GetY1(),SegBox->GetX1()+(SegBox->GetX2()-SegBox->GetX1())/(max-min)*(OnSeg->GetEnd(s)-min),SegBox->GetY2(),0,"NDC");
	OnBox[s]->SetFillColor(3);
	nonbox++;
      }
      delete OnSeg;

      // SEL
      for(int j=0; j<nselbox; j++) delete SelBox[j];
      nselbox=0; delete SelBox;
      if(SelSeg!=NULL&&SelSeg->GetStatus()){
	OnSeg = new Segments(min,max);
	if(aInSeg!=NULL&&aInSeg->GetStatus()) OnSeg->Intersect(aInSeg);
	OnSeg->Intersect(SelSeg);
	OnSeg->Intersect(ReadTriggerSegments::Seg);
	SelBox = new TPave* [OnSeg->GetNsegments()];
	for(int s=0; s<OnSeg->GetNsegments(); s++){
	  SelBox[s]= new TPave(SegBox->GetX1()+(SegBox->GetX2()-SegBox->GetX1())/(max-min)*(OnSeg->GetStart(s)-min),SegBox->GetY1(),SegBox->GetX1()+(SegBox->GetX2()-SegBox->GetX1())/(max-min)*(OnSeg->GetEnd(s)-min),SegBox->GetY2(),0,"NDC");
	  SelBox[s]->SetFillColor(kGreen+3);
	  nselbox++;
	}
	delete OnSeg;
      }

    }

 
  }

  // loudest plot
  Gfreqtimeloud->RemovePoint(0);
  Gfreqtimeloud2->RemovePoint(0);
  if(smax>0){
    Gfreqtimeloud->SetPoint(0, tmax, fmax);
    Gfreqtimeloud2->SetPoint(0, tmax, fmax);
  }


  return true;
}

////////////////////////////////////////////////////////////////////////////////////
bool TriggerPlot::PrintPlot(const string aPlotType, const int aCollIndex, const bool aFirst){
////////////////////////////////////////////////////////////////////////////////////
  vector <int> ind(1,aCollIndex);
  return PrintPlots(aPlotType,ind,aFirst);
}

////////////////////////////////////////////////////////////////////////////////////
bool TriggerPlot::PrintPlots(const string aPlotType, const vector<int> aCollIndexes, const bool aFirst){
////////////////////////////////////////////////////////////////////////////////////

  if(aFirst){
    ResetLegend();
    Wpad->Clear();
  }

  // locals
  struct tm utcmin;
  char buffer [80];
  stringstream tmpstream;
  string datemin, stmin;
  
  for(int c=0; c<(int)aCollIndexes.size(); c++){
    
    if(aCollIndexes[c]<0||aCollIndexes[c]>=Ncoll){
      cerr<<"TriggerPlot::PrintPlots: Collection #"<<aCollIndexes[c]<<" does not exist"<<endl;
      continue;
    }

    // time start
    GPSToUTC(&utcmin, (int)CollectionParam[c]->GetTimeMin());
    strftime (buffer,80,"%Y-%b-%d %H:%M:%S",&utcmin);
    tmpstream.flags ( ios::fixed );
    tmpstream<<setprecision(3)<<CollectionParam[c]->GetTimeMin();
    datemin=(string)buffer;
    stmin=tmpstream.str();
    tmpstream.clear(); tmpstream.str("");
    AddText("",0.01,0.01,0.04);
    
    // FREQTIME
    if(!aPlotType.compare("freqtime")){
      if(!c){ 
	if(aFirst) Draw(Hfreqtime[aCollIndexes[c]],"");
	if(tb){
	  Draw(SegBox,"same");
	  for(int i=0; i<nonbox; i++) Draw(OnBox[i],"same");
	  for(int i=0; i<nselbox; i++) Draw(SelBox[i],"same");
	}
	SetGridx(1); SetGridy(1); SetLogx(0); SetLogy(1);
	Hfreqtime[aCollIndexes[c]]->GetYaxis()->SetMoreLogLabels();
      }
      if(Gfreqtime[aCollIndexes[c]]->GetN()) Draw(Gfreqtime[aCollIndexes[c]],"Psame");
      if(plot_star&&c==(int)aCollIndexes.size()-1&&Gfreqtimeloud->GetN()){
	Draw(Gfreqtimeloud,"Psame");
	Draw(Gfreqtimeloud2,"Psame");
      }
      AddLegendEntry(Gfreqtime[aCollIndexes[c]],Slegend[aCollIndexes[c]],"P");
        if(Pusedate) UpdateText("Start: "+datemin+" UTC");
	else UpdateText("Start: "+stmin);
    }

    // SNRTIME
    else if(!aPlotType.compare("snrtime")){
      if(!c){ 
	if(aFirst) Draw(Hsnrtime[aCollIndexes[c]],"");
	if(tb){
	  Draw(SegBox,"same");
	  for(int i=0; i<nonbox; i++) Draw(OnBox[i],"same");
	  for(int i=0; i<nselbox; i++) Draw(SelBox[i],"same");
	}
	SetGridx(1); SetGridy(1); SetLogx(0); SetLogy(1);
      }
      if(Gsnrtime[aCollIndexes[c]]->GetN()) Draw(Gsnrtime[aCollIndexes[c]],"Psame");
      AddLegendEntry(Gsnrtime[aCollIndexes[c]],Slegend[aCollIndexes[c]],"P");
      if(Pusedate) UpdateText("Start: "+datemin+" UTC");
      else UpdateText("Start: "+stmin);
    }

    // SNRFREQ
    else if(!aPlotType.compare("snrfreq")){
      if(!c){ 
	if(aFirst) Draw(Hsnrfreq[aCollIndexes[c]],"");
	SetGridx(1); SetGridy(1); SetLogx(1); SetLogy(1);
	Hsnrfreq[aCollIndexes[c]]->GetXaxis()->SetMoreLogLabels();
      }
      if(Gsnrfreq[aCollIndexes[c]]->GetN()) Draw(Gsnrfreq[aCollIndexes[c]],"Psame");
      AddLegendEntry(Gsnrfreq[aCollIndexes[c]],Slegend[aCollIndexes[c]],"P");
      UpdateText("");
    }

    // RATE
    else if(!aPlotType.compare("rate")){
      if(!c){ 
	if(aFirst) Draw(Htime[aCollIndexes[c]],"P");
	else Draw(Htime[aCollIndexes[c]],"Psame");
	Htime[aCollIndexes[c]]->GetYaxis()->SetRangeUser(0.5/Htime[aCollIndexes[c]]->GetBinWidth(0),2*Htime[aCollIndexes[c]]->GetMaximum()+1);
	if(tb){
	  Draw(SegBox,"same");
	  for(int i=0; i<nonbox; i++) Draw(OnBox[i],"same");
	  for(int i=0; i<nselbox; i++) Draw(SelBox[i],"same");
	}
	SetGridx(1); SetGridy(1); SetLogx(0); SetLogy(1);
      }
      Draw(Htime[aCollIndexes[c]],"Psame");
      AddLegendEntry(Htime[aCollIndexes[c]],Slegend[aCollIndexes[c]],"P");
      if(Pusedate) UpdateText("Start: "+datemin+" UTC");
      else UpdateText("Start: "+stmin);
    }
 
    // FREQUENCY
    else if(!aPlotType.compare("frequency")){
      if(!c){ 
	if(aFirst) Draw(Hfreq[aCollIndexes[c]],"");
	else Draw(Hfreq[aCollIndexes[c]],"same");
	Hfreq[aCollIndexes[c]]->GetYaxis()->SetRangeUser(0.5,2*Hfreq[aCollIndexes[c]]->GetMaximum()+1);
	SetGridx(1); SetGridy(1); SetLogx(1); SetLogy(1);
      }
      Draw(Hfreq[aCollIndexes[c]],"same");
      AddLegendEntry(Hfreq[aCollIndexes[c]],Slegend[aCollIndexes[c]],"L");
      UpdateText("");
    }

    // SNR
    else if(!aPlotType.compare("snr")){
      if(!c){ 
	if(aFirst) Draw(Hsnr[aCollIndexes[c]],"");
	else Draw(Hsnr[aCollIndexes[c]],"same");
	Hsnr[aCollIndexes[c]]->GetYaxis()->SetRangeUser(0.5,2*Hsnr[aCollIndexes[c]]->GetMaximum()+1);
	SetGridx(1); SetGridy(1); SetLogx(1); SetLogy(1);
      }
      Draw(Hsnr[aCollIndexes[c]],"same");
      AddLegendEntry(Hsnr[aCollIndexes[c]],Slegend[aCollIndexes[c]],"L");
      UpdateText("");
    }
    else{
      cerr<<"TriggerPlot::PrintPlots: the plot type "<<aPlotType<<" is not supported"<<endl;
      return false;
    }
  }

  return true;
}

////////////////////////////////////////////////////////////////////////////////////
bool TriggerPlot::PrintCollectionPlot(const string aPlotType){
////////////////////////////////////////////////////////////////////////////////////
  vector <int> ind;
  for(int i=0; i<Ncoll; i++) ind.push_back(i);
  return PrintPlots(aPlotType,ind);
}


////////////////////////////////////////////////////////////////////////////////////
bool TriggerPlot::PrintCollectionPanel(void){
////////////////////////////////////////////////////////////////////////////////////
  ResetLegend();
  Wpad->Clear();
  GwollumPlot::DivideCanvas(6);

  for(int i=0; i<Ncoll; i++){
    
    if(!i){ 
      Draw(Hfreqtime[i],"",1);
      if(tb){
	Draw(SegBox,"same",1);
	for(int ii=0; ii<nonbox; ii++) Draw(OnBox[ii],"same",1);
	for(int ii=0; ii<nselbox; ii++) Draw(SelBox[ii],"same",1);
      }
      SetGridx(1,1); SetGridy(1,1); SetLogx(0,1); SetLogy(1,1);
      Draw(Hsnrtime[i],"",2);
      if(tb){
	Draw(SegBox,"same",2);
	for(int ii=0; ii<nonbox; ii++) Draw(OnBox[ii],"same",2);
	for(int ii=0; ii<nselbox; ii++) Draw(SelBox[ii],"same",2);
      }
      SetGridx(1,2); SetGridy(1,2); SetLogx(0,2); SetLogy(1,2);
      Draw(Hsnrfreq[i],"",3);
      SetGridx(1,3); SetGridy(1,3); SetLogx(1,3); SetLogy(1,3);
      Draw(Htime[i],"P",4);
      Htime[i]->GetYaxis()->SetRangeUser(0.5/Htime[i]->GetBinWidth(0),2*Htime[i]->GetMaximum()+1);
      if(tb){      
	Draw(SegBox,"same",4);
	for(int ii=0; ii<nonbox; ii++) Draw(OnBox[ii],"same",4);
	for(int ii=0; ii<nselbox; ii++) Draw(SelBox[ii],"same",4);
      }
      SetGridx(1,4); SetGridy(1,4); SetLogx(0,4); SetLogy(1,4);
      Draw(Hfreq[i],"",5);
      Hfreq[i]->GetYaxis()->SetRangeUser(0.5,2*Hfreq[i]->GetMaximum()+1);
      SetGridx(1,5); SetGridy(1,5); SetLogx(1,5); SetLogy(1,5);
      Draw(Hsnr[i],"",6);
      Hsnr[i]->GetYaxis()->SetRangeUser(0.5,2*Hsnr[i]->GetMaximum()+1);
      SetGridx(1,6); SetGridy(1,6); SetLogx(1,6); SetLogy(1,6);
    }
    
    if(Gfreqtime[i]->GetN()) Draw(Gfreqtime[i],"Psame",1);
    if(plot_star&&i==Ncoll-1&&Gfreqtimeloud->GetN()){
      Draw(Gfreqtimeloud,"Psame",1);
      Draw(Gfreqtimeloud2,"Psame",1);
    }
    if(Gsnrtime[i]->GetN())  Draw(Gsnrtime[i],"Psame",2);
    if(Gsnrfreq[i]->GetN()) Draw(Gsnrfreq[i],"Psame",3);
    Draw(Htime[i],"same",4);
    Draw(Hfreq[i],"same",5);
    Draw(Hsnr[i],"same",6);
    
    AddLegendEntry(Gsnrtime[i],Slegend[i],"PL");// one legend per collection
  }

  return true;
}

////////////////////////////////////////////////////////////////////////////////////
bool TriggerPlot::PrintProcPlot(const string aPlotType, const string aProcType, const string aProcName, const vector<int> aCollIndexes, const bool aFirst){
////////////////////////////////////////////////////////////////////////////////////
  if(!aCollIndexes.size()){
    cerr<<"TriggerPlot::PrintProcPlot: at least one collection is required"<<endl;
    return false;
  }
  for(int i=0; i<(int)aCollIndexes.size(); i++)
    if(aCollIndexes[i]<0||aCollIndexes[i]>=Ncoll){
      cerr<<"TriggerPlot::PrintProcPlot: Collection #"<<aCollIndexes[i]<<" does not exist"<<endl;
      return false;
    }

  // plot selection
  TH1D **htmp;
  if(!aPlotType.compare("rate")) htmp = Htime;
  else if(!aPlotType.compare("frequency")) htmp = Hfreq;
  else if(!aPlotType.compare("snr")) htmp = Hsnr;
  else{
    cerr<<"TriggerPlot::PrintProcPlot: the plot type "<<aPlotType<<" is not supported"<<endl;
    return false;
  }
  
  // make proc
  if(!aProcType.compare("SUM")){
    delete Hproc; Hproc = new TH1D(*htmp[aCollIndexes[0]]); Hproc->SetName(aProcType.c_str());
    for(int i=1; i<(int)aCollIndexes.size(); i++) Hproc->Add(htmp[aCollIndexes[i]]);
  }
  else{
    cerr<<"TriggerPlot::PrintProcPlot: the process type "<<aProcType<<" is not supported"<<endl;
    return false;
  }
  
  // hard-coded style 
  Hproc->SetTitle(htmp[aCollIndexes[0]]->GetTitle());
  Hproc->SetLineWidth(2);
  Hproc->SetLineColor(46);
  
  // draw proc plot
  if(aFirst){
    ResetLegend();
    Wpad->Clear();
    if(!aPlotType.compare("rate")){
      Hproc->GetYaxis()->SetRangeUser(0.5/GetNTriggers(),2*Hproc->GetMaximum()+1);
      SetGridx(1); SetGridy(1); SetLogx(0); SetLogy(1);
    }
    else{
      Hproc->GetYaxis()->SetRangeUser(0.5,2*Hproc->GetMaximum()+1);
      SetGridx(1); SetGridy(1); SetLogx(1); SetLogy(1);
    }

    Draw(Hproc,"");
  }
  else Draw(Hproc,"same");
  AddLegendEntry(Hproc,aProcName,"L");
  
  return true;
}


////////////////////////////////////////////////////////////////////////////////////
void TriggerPlot::SetDateFormat(const bool aDateFormat){
////////////////////////////////////////////////////////////////////////////////////
  Pusedate=aDateFormat;
  for(int i=0; i<Ncoll; i++) ApplyStyle(i);
  return;
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerPlot::SetSelectionSegments(Segments *SelectionSegments){
////////////////////////////////////////////////////////////////////////////////////

  // no selection
  if(SelectionSegments==NULL || !SelectionSegments->GetStatus()){
    SelSeg=NULL;
    return;
  }

  // new segments
  SelSeg=SelectionSegments;
  
  return;
}

////////////////////////////////////////////////////////////////////////////////////
bool TriggerPlot::SetCollectionLegend(const int aCollIndex, const string aLegend){
////////////////////////////////////////////////////////////////////////////////////
  if(aCollIndex<0||aCollIndex>=Ncoll){
    cerr<<"TriggerPlot::SetCollectionLegend: Collection #"<<aCollIndex<<" does not exist"<<endl;
    return false;
  }
  Slegend[aCollIndex]=aLegend;
  return true;
}

////////////////////////////////////////////////////////////////////////////////////
bool TriggerPlot::SetCollectionColor(const int aCollIndex, const int aColorIndex){
////////////////////////////////////////////////////////////////////////////////////
  if(aCollIndex<0||aCollIndex>=Ncoll){
    cerr<<"TriggerPlot::SetCollectionColor: Collection #"<<aCollIndex<<" does not exist"<<endl;
    return false;
  }
  TPcolor[aCollIndex]=aColorIndex;
  return true;
}

////////////////////////////////////////////////////////////////////////////////////
bool TriggerPlot::SetCollectionMarker(const int aCollIndex, const int aMarkerStyleIndex, const double aMarkerSizeIndex){
////////////////////////////////////////////////////////////////////////////////////
  if(aCollIndex<0||aCollIndex>=Ncoll){
    cerr<<"TriggerPlot::SetCollectionMarkerStyle: Collection #"<<aCollIndex<<" does not exist"<<endl;
    return false;
  }
  TPmstyle[aCollIndex] = aMarkerStyleIndex;
  TPmsize[aCollIndex]  = aMarkerSizeIndex;
  return true;
}

////////////////////////////////////////////////////////////////////////////////////
bool TriggerPlot::SetCollectionUseClusters(const int aCollIndex, const int aUseClusters){
////////////////////////////////////////////////////////////////////////////////////
  if(aCollIndex<0||aCollIndex>=Ncoll){
    cerr<<"TriggerPlot::SetCollectionUseClusters: Collection #"<<aCollIndex<<" does not exist"<<endl;
    return false;
  }
  Suseclusters[aCollIndex]=aUseClusters;
  return true;
}

////////////////////////////////////////////////////////////////////////////////////
TriggerSelect*  TriggerPlot::GetCollectionSelection(const int aCollIndex){
////////////////////////////////////////////////////////////////////////////////////
  if(aCollIndex<0||aCollIndex>=Ncoll){
    cerr<<"TriggerPlot::GetCollectionSelection: Collection #"<<aCollIndex<<" does not exist"<<endl;
    return NULL;
  }
 
  return CollectionParam[aCollIndex];
}

////////////////////////////////////////////////////////////////////////////////////
int TriggerPlot::GetCollectionNClusters(const int aCollIndex){
////////////////////////////////////////////////////////////////////////////////////
  if(aCollIndex<0||aCollIndex>=Ncoll){
    cerr<<"TriggerPlot::GetCollectionNClusters: Collection #"<<aCollIndex<<" does not exist"<<endl;
    return 0;
  }
  return Gfreqtime[aCollIndex]->GetN();
}

////////////////////////////////////////////////////////////////////////////////////
double TriggerPlot::GetCollectionTotalEventDuration(const int aCollIndex){
////////////////////////////////////////////////////////////////////////////////////
  if(aCollIndex<0||aCollIndex>=Ncoll){
    cerr<<"TriggerPlot::GetCollectionTotalEventDuration: Collection #"<<aCollIndex<<" does not exist"<<endl;
    return 0;
  }
  return ev_duration_tot[aCollIndex];
}

////////////////////////////////////////////////////////////////////////////////////
TH1D* TriggerPlot::GetTH1D(const string aPlotType, const int aCollIndex){
////////////////////////////////////////////////////////////////////////////////////
  if(aCollIndex<0||aCollIndex>=Ncoll){
    cerr<<"TriggerPlot::GetTH1D: Collection #"<<aCollIndex<<" does not exist"<<endl;
    return NULL;
  }
  if(!aPlotType.compare("rate")) return (TH1D*)(Htime[aCollIndex]->Clone());
  if(!aPlotType.compare("snr")) return (TH1D*)(Hsnr[aCollIndex]->Clone());
  if(!aPlotType.compare("frequency")) return (TH1D*)(Hfreq[aCollIndex]->Clone());
  cerr<<"TriggerPlot::GetTH1D: plot type "<<aPlotType<<" does not exist"<<endl;
  return NULL;
}

////////////////////////////////////////////////////////////////////////////////////
double TriggerPlot::GetYmax(const string aPlotType, const int aCollIndex){
////////////////////////////////////////////////////////////////////////////////////
  if(aCollIndex<0||aCollIndex>=Ncoll){
    cerr<<"TriggerPlot::GetYmax: Collection #"<<aCollIndex<<" does not exist"<<endl;
    return 1.0;
  }
  
  if(!aPlotType.compare("rate")) return Htime[aCollIndex]->GetYaxis()->GetXmax();
  if(!aPlotType.compare("snr")) return Hsnr[aCollIndex]->GetYaxis()->GetXmax();
  if(!aPlotType.compare("frequency")) return Hfreq[aCollIndex]->GetYaxis()->GetXmax();
  if(!aPlotType.compare("snrtime")) return Hsnrtime[aCollIndex]->GetYaxis()->GetXmax();
  if(!aPlotType.compare("freqtime")) return Hfreqtime[aCollIndex]->GetYaxis()->GetXmax();
  if(!aPlotType.compare("snrfreq")) return Hsnrfreq[aCollIndex]->GetYaxis()->GetXmax();

  return 1.0;
}

////////////////////////////////////////////////////////////////////////////////////
double TriggerPlot::GetYmin(const string aPlotType, const int aCollIndex){
////////////////////////////////////////////////////////////////////////////////////
  if(aCollIndex<0||aCollIndex>=Ncoll){
    cerr<<"TriggerPlot::GetYmin: Collection #"<<aCollIndex<<" does not exist"<<endl;
    return 0.0;
  }
  
  if(!aPlotType.compare("rate")) return Htime[aCollIndex]->GetYaxis()->GetXmin();
  if(!aPlotType.compare("snr")) return Hsnr[aCollIndex]->GetYaxis()->GetXmin();
  if(!aPlotType.compare("frequency")) return Hfreq[aCollIndex]->GetYaxis()->GetXmin();
  if(!aPlotType.compare("snrtime")) return Hsnrtime[aCollIndex]->GetYaxis()->GetXmin();
  if(!aPlotType.compare("freqtime")) return Hfreqtime[aCollIndex]->GetYaxis()->GetXmin();
  if(!aPlotType.compare("snrfreq")) return Hsnrfreq[aCollIndex]->GetYaxis()->GetXmin();

  return 0.0;
}
