//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#include "Chain.h"

ClassImp(Chain)
Chain::Chain(const char* chainname, const char* pattern, const bool verbose) : TChain(chainname) {
  fchainname=(string)chainname;
  fverbose=verbose;
  rootfilename.clear();
  spattern=(string)pattern;

  vector <string> sub_pattern=SplitString(spattern,' ');

  if(sub_pattern.size()){
    if((sub_pattern[0].substr(0,5)).compare("root:")) ChainPOSIX();
    else ChainXROOTD(sub_pattern);
  }

  sub_pattern.clear();
}

Chain::~Chain(void){}

void Chain::ChainPOSIX(void){

  string ls;
  ls = "ls -U "+spattern;
  
  if(fverbose) cout<<"Opening root files in "<<spattern<<"..."<<endl;
    
  FILE *in;
  in = popen(ls.c_str(), "r");

  char filename[100000];
  while(fscanf(in,"%s",filename) != EOF){
    if(fverbose) cerr<<filename<<" ... ";
    TFile *tmp = new TFile(filename);
    if( !tmp || tmp->IsZombie() ){
      if(fverbose) cerr<<"not a root file"<<endl;
      continue;
    }
    if( tmp->ReadKeys() ){
      tmp->Close();
      if(int retcode = TChain::AddFile(filename,-1)){
        if(fverbose) cerr<<"OK! "<<retcode<<endl;
	rootfilename.push_back((string)filename);
      }
      else if(fverbose) cerr<<fchainname<<" not found"<<endl;
    }
    else 
      if(fverbose) cerr<<"No Keys !"<<endl;
  }
  pclose(in);
  return;
}

void Chain::ChainXROOTD(const vector <string> rootfiles){
 
  for(int f=0; f<(int)rootfiles.size(); f++){
    if(fverbose) cerr<<rootfiles[f]<<" ... ";
    TFile *tmp = TFile::Open(rootfiles[f].c_str());
    if( !tmp || tmp->IsZombie() ){
      if(fverbose) cerr<<"not a root file"<<endl;
      continue;
    }
    if( tmp->ReadKeys() ){
      tmp->Close();
      if(int retcode = TChain::AddFile(rootfiles[f].c_str(),-1)){
        if(fverbose) cerr<<"OK! "<<retcode<<endl;
	rootfilename.push_back(rootfiles[f]);
      }
      else if(fverbose) cerr<<fchainname<<" not found"<<endl;
    }
    else 
      if(fverbose) cerr<<"No Keys !"<<endl;
  }
  return;
}

bool Chain::AddFile(const string afilename){
  if(fverbose) cout<<afilename<<" ... ";
  TFile tmp(afilename.c_str());
  if( tmp.IsZombie() ){
    if(fverbose) cerr<<"not a root file"<<endl;
    return false;
  }
  if( tmp.ReadKeys() ){
    tmp.Close();
    if(int retcode = TChain::AddFile(afilename.c_str(),-1)){
      if(fverbose) cout<<"OK! "<<retcode<<endl;
      rootfilename.push_back(afilename);
    }
    else{
      if(fverbose) cout<<fchainname<<" not found"<<endl;
      return false;
    }
  }
  else{ 
    if(fverbose) cout<<"No Keys !"<<endl;
    return false;
  }

  return true;
}

string Chain::GetFilePath(const int afilenumber){
  if(afilenumber<0||afilenumber>=(int)rootfilename.size()){
    cerr<<"Chain::GetFilePath: filenumber does not exist"<<endl;
    return "none";
  }
  return rootfilename[afilenumber];
}
