//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#include "FFT.h"

////////////////////////////////////////////////////////////////////////////////////
fft2d::fft2d(const int aSize_x, const int aSize_y, const string aPlan){
////////////////////////////////////////////////////////////////////////////////////
 
  // parameters
  fSize_x=fabs(aSize_x);
  fSize_y=fabs(aSize_y);
  fPlan=aPlan;
  
  // check sizes
  if(fSize_x<2) fSize_x=2;
  fSize_x=fSize_x/2*2;// must be even
  if(fSize_y<2) fSize_y=2;
  fSize_y=fSize_y/2*2;// must be even

  // matrix size
  n_xy=fSize_x*fSize_y;
  
  // frequency-domain size
  fSize_fx=fSize_x;
  fSize_fy=fSize_y/2+1;
  n_fxy=fSize_fx*fSize_fy;

  // allocate memory for vectors
  v_xy = new double [n_xy];
  for(int i=0; i<n_xy; i++) v_xy[i]=0.0;
  v_fxy = new fftw_complex [n_fxy];
  for(int i=0; i<n_fxy; i++){
    v_fxy[i][0]=0.0;
    v_fxy[i][1]=0.0;
  }

  // FFT plans
  if(!fPlan.compare("FFTW_MEASURE")){
    FFT_Forward  = fftw_plan_dft_r2c_2d(fSize_x, fSize_y, v_xy, v_fxy, FFTW_MEASURE);
    FFT_Backward = fftw_plan_dft_c2r_2d(fSize_x, fSize_y, v_fxy, v_xy, FFTW_MEASURE);
  }
  else if(!fPlan.compare("FFTW_PATIENT")){
    FFT_Forward  = fftw_plan_dft_r2c_2d(fSize_x, fSize_y, v_xy, v_fxy, FFTW_PATIENT);
    FFT_Backward = fftw_plan_dft_c2r_2d(fSize_x, fSize_y, v_fxy, v_xy, FFTW_PATIENT);
  }
  else{
    FFT_Forward  = fftw_plan_dft_r2c_2d(fSize_x, fSize_y, v_xy, v_fxy, FFTW_ESTIMATE);
    FFT_Backward = fftw_plan_dft_c2r_2d(fSize_x, fSize_y, v_fxy, v_xy, FFTW_ESTIMATE);
  }
     
}


////////////////////////////////////////////////////////////////////////////////////
fft2d::~fft2d(void){
////////////////////////////////////////////////////////////////////////////////////
  fftw_destroy_plan(FFT_Forward);
  fftw_destroy_plan(FFT_Backward);
  delete v_xy;
  delete v_fxy;
}
